import csv
import os
from datetime import datetime

from depwatch.history import History


def write_histories(filename: str, histories: list[History]):
    dirname = str(os.environ.get("OUTPUT_DIR") or "")
    dirname = dirname if dirname != "" else "."

    with open(dirname + "/" + filename, "w", newline="") as csvfile:
        writer = csv.writer(csvfile, delimiter=",")
        writer.writerow(History.keys())
        for h in histories:
            formatted_values = []
            for v in h.values():
                if isinstance(v, datetime):
                    formatted_values.append(v.isoformat())
                else:
                    formatted_values.append(v)

            writer.writerow(formatted_values)



from datetime import datetime
from depwatch.history import (
    DeploymentHistory,
    History,
    RepositoryHistory,
    create_histories,
)


class TestHistory:
    def test_create_histories(self):
        repository_histories = [
            RepositoryHistory(
                datetime.fromisoformat("2023-02-25T20:00:00+00:00"),
                datetime.fromisoformat("2023-02-26T22:00:00+00:00"),
                "8eddb4b87a1460b29bce25448623b270465e9715",
            ),
            RepositoryHistory(
                datetime.fromisoformat("2023-02-23T10:05:00+00:00"),
                datetime.fromisoformat("2023-02-23T11:15:00+00:00"),
                "12d4c5f2a123b6fb77eccb57be1be3a29438fadb",
            ),
            RepositoryHistory(
                datetime.fromisoformat("2023-02-23T10:00:00+00:00"),
                datetime.fromisoformat("2023-02-23T11:00:00+00:00"),
                "cbd1519fd8b7daff2655346676065f844a2ef3df",
            ),
            RepositoryHistory(
                datetime.fromisoformat("2023-02-22T14:00:00+00:00"),
                datetime.fromisoformat("2023-02-22T17:00:00+00:00"),
                "b25ab2d8590fc678ced72fe378c0bf2dc04da15c",
            ),
        ]
        deployment_histories = [
            DeploymentHistory(
                datetime.fromisoformat("2023-02-27T10:15:00+00:00"),
                "8eddb4b87a1460b29bce25448623b270465e9715",
            ),
            DeploymentHistory(
                datetime.fromisoformat("2023-02-24T09:30:00+00:00"),
                "b25ab2d8590fc678ced72fe378c0bf2dc04da15c",
            ),
            DeploymentHistory(
                datetime.fromisoformat("2023-02-23T11:30:00+00:00"),
                "12d4c5f2a123b6fb77eccb57be1be3a29438fadb",
            ),
        ]

        histories = create_histories(repository_histories, deployment_histories)

        assert histories == [
            History(
                datetime.fromisoformat("2023-02-25T20:00:00+00:00"),
                datetime.fromisoformat("2023-02-26T22:00:00+00:00"),
                datetime.fromisoformat("2023-02-27T10:15:00+00:00"),
            ),
            History(
                datetime.fromisoformat("2023-02-23T10:05:00+00:00"),
                datetime.fromisoformat("2023-02-23T11:15:00+00:00"),
                datetime.fromisoformat("2023-02-23T11:30:00+00:00"),
            ),
            History(
                datetime.fromisoformat("2023-02-23T10:00:00+00:00"),
                datetime.fromisoformat("2023-02-23T11:00:00+00:00"),
                None,
            ),
            History(
                datetime.fromisoformat("2023-02-22T14:00:00+00:00"),
                datetime.fromisoformat("2023-02-22T17:00:00+00:00"),
                datetime.fromisoformat("2023-02-24T09:30:00+00:00"),
            ),
        ]



from github import Github
import os
from datetime import datetime, timezone
from depwatch.exception import DepwatchException

from depwatch.history import RepositoryHistory


def get_main_branch(name: str) -> str:
    gh = Github(os.environ.get("GITHUB_ACCESS_TOKEN"))

    repo = gh.get_repo(name)
    branches = repo.get_branches()

    if "main" in [b.name for b in branches]:
        return "main"
    elif "master" in [b.name for b in branches]:
        return "master"
    else:
        raise DepwatchException("'main' or 'master' branch was not found")


def get_repository_history(name: str, base: str, limit: int) -> list[RepositoryHistory]:
    histories = []

    gh = Github(os.environ.get("GITHUB_ACCESS_TOKEN"), per_page=100)
    repo = gh.get_repo(name)
    pulls = repo.get_pulls(state="closed", base=base)[:limit]

    for p in pulls:
        if p.merged_at is None:
            continue

        first_committed_at: datetime = p.get_commits()[0].commit.author.date.replace(
            tzinfo=timezone.utc
        )
        merged_at: datetime = p.merged_at.replace(tzinfo=timezone.utc)
        merge_commit_sha: str = p.merge_commit_sha

        histories.append(
            RepositoryHistory(first_committed_at, merged_at, merge_commit_sha)
        )

    return histories
